!---------------------------------------------------------------!
! This program is to convert ERA-interim data to CReSS data.    !
!                                                               !
! developed : Satoki Tsujino                                    !
!---------------------------------------------------------------!

program NCEP_convert

  use Phys_Const
  use file_operate
  use Statistics
  use gtool_history
  use Thermo_Function
  use Basis

  implicit none

  real, parameter :: undefc=-1.0e35

!-- For NetCDF, each pointer

  integer(2), pointer, dimension(:) :: s1
  integer, pointer, dimension(:) :: i1
  real, pointer, dimension(:) :: r1

!-- internal variables

  integer :: undef
  integer :: i, j, k, id, jd, ct, ir
  integer :: nx, ny, nt, nc, ns

  integer :: land_flag(2)   ! land_flag(1) = ocean, land_flag(2) = land
  integer :: nx1, ny1, nxt, nyt, nxs, nys, nts, nz, ipflag
  integer, allocatable, dimension(:,:,:) :: tmp1
  integer, allocatable, dimension(:,:) :: tmps1, tmpt
  integer, allocatable, dimension(:) :: zi, ti
  real :: pfact, gfact, rhfact
  real, allocatable, dimension(:) :: lon, lat, lon1, lat1, lont, z
  real, allocatable, dimension(:) :: lons, lats, latt, lattr
  real, allocatable, dimension(:,:) :: tmptr, tmps, ter, land
  real, allocatable, dimension(:,:) :: vanish2d
  real, allocatable, dimension(:,:,:) :: tmp2
  real, allocatable, dimension(:,:) :: tmps2
  real, allocatable, dimension(:,:,:) :: val2
  real, allocatable, dimension(:,:,:,:) :: val3
  double precision :: factor, addoff
  logical :: rhflag   ! moisture is RH (.true.) or mixing ratio (.false.).
  logical :: pflag    ! pressure is hPa (.true.) or Pa (.false.).
  logical :: gphflag  ! geopotential is gz (m^2s^-2, .true.) or GPH (m, .false.)
  logical :: lonrev_flag, latrev_flag, levrev_flag
  logical :: lontrev_flag, lattrev_flag, lonsrev_flag, latsrev_flag
  type(dtime) :: stime, etime

!-- z, u, v, p, t, w, q, s
  character(100), dimension(8) :: vname, ssvname
  character(100) :: uvname, vvname, wvname, pvname, tvname, qvname, zvname, svname
  character(100) :: usname, vsname, wsname, psname, tsname, qsname, ssname
  character(100) :: lonname, latname, levname, tname, tername, tstname, tttname
  character(100) :: factname, undefname, addname
  character(100) :: lonsname, latsname, lontname, lattname, landname
  character(1000) :: ofoot, atmfile, terfile, surfile
  character(1000) :: oname
  character(100) :: forma

  namelist /basic /atmfile, uvname, vvname, wvname, pvname, tvname, qvname,  &
  &                zvname, svname, lonname, latname, levname, tname,  &
  &                factname, addname, undefname, ipflag, gphflag, pflag, rhflag
  namelist /terrain /terfile, tername, landname, land_flag, lontname,  &
  &                  lattname, tttname
  namelist /surface /surfile, usname, vsname, wsname, psname, tsname, qsname,  &
  &                  ssname, lonsname, latsname, tstname

  read(5,nml=basic)
  read(5,nml=terrain)
  read(5,nml=surface)

  stime%year_d=1900
  stime%month_d=1
  stime%day_d=1
  stime%hour_d=0
  stime%min_d=0
  stime%sec_d=0

  vname(1)=zvname
  vname(2)=uvname
  vname(3)=vvname
  vname(4)=pvname
  vname(5)=tvname
  vname(6)=wvname
  vname(7)=qvname
  vname(8)=svname

  ssvname(1)=''
  ssvname(2)=usname
  ssvname(3)=vsname
  ssvname(4)=psname
  ssvname(5)=tsname
  ssvname(6)=wsname
  ssvname(7)=qsname
  ssvname(8)=ssname

!-- reading each parameter from NetCDF file

  nullify(r1)
  call HistoryGetpointer( trim(adjustl(atmfile)), trim(adjustl(lonname)), r1 )
  nx1=size(r1)
  deallocate(r1)

  nullify(r1)
  call HistoryGetpointer( trim(adjustl(atmfile)), trim(adjustl(latname)), r1 )
  ny1=size(r1)
  deallocate(r1)

  nullify(i1)
  call HistoryGetpointer( trim(adjustl(atmfile)), trim(adjustl(levname)), i1 )
  nz=size(i1)
  deallocate(i1)

  nullify(i1)
  call HistoryGetpointer( trim(adjustl(atmfile)), trim(adjustl(tname)), i1 )
  nt=size(i1)
  deallocate(i1)

  if(len_trim(adjustl(terfile))/=0)then
     nullify(r1)
     call HistoryGetpointer( trim(adjustl(terfile)), trim(adjustl(lontname)), r1 )
     nxt=size(r1)
     deallocate(r1)

     nullify(r1)
     call HistoryGetpointer( trim(adjustl(terfile)), trim(adjustl(lattname)), r1 )
     nyt=size(r1)
     deallocate(r1)
  end if

  if(len_trim(adjustl(surfile))/=0)then
     nullify(r1)
     call HistoryGetpointer( trim(adjustl(surfile)), trim(adjustl(lontname)), r1 )
     nxs=size(r1)
     deallocate(r1)

     nullify(r1)
     call HistoryGetpointer( trim(adjustl(surfile)), trim(adjustl(lattname)), r1 )
     nys=size(r1)
     deallocate(r1)

     nullify(i1)
     call HistoryGetpointer( trim(adjustl(surfile)), trim(adjustl(tstname)), i1 )
     nts=size(i1)
     deallocate(i1)
  end if

!-- setting each variable

  select case (ipflag)
  case (1)
     nx=nx1
     ny=ny1
  case (3)
     nx=nxs
     ny=nys
  case (4)
     nx=nxt
     ny=nyt
  end select

!-- allocating some variables

  allocate(lon(nx))
  allocate(lat(ny))
  allocate(lon1(nx1))
  allocate(lat1(ny1))
  allocate(zi(nz))
  allocate(z(nz))
  allocate(ti(nt))

  if(len_trim(adjustl(surfile))/=0)then
     allocate(lons(nxs))
     allocate(lats(nys))
     allocate(tmps1(nxs,nys))
     allocate(tmps2(nxs,nys))
  end if

  if(len_trim(adjustl(terfile))/=0)then
     allocate(lont(nxt))
     allocate(latt(nyt))
     allocate(tmpt(nxt,nyt))
     allocate(tmptr(nxt,nyt))
  end if

  allocate(tmp1(nx1,ny1,nz))
  allocate(tmp2(nx1,ny1,nz))
  allocate(val2(nx,ny,8))
  allocate(val3(nx,ny,nz,8))
  allocate(ter(nx,ny))
  allocate(land(nx,ny))
  allocate(vanish2d(nx,ny))

  val2=0.0
  val3=0.0
  vanish2d=0.0

!-- reading each axis data from NetCDF file

  call HistoryGet( trim(adjustl(atmfile)), trim(adjustl(lonname)), lon1 )
  call HistoryGet( trim(adjustl(atmfile)), trim(adjustl(latname)), lat1 )
  call HistoryGet( trim(adjustl(atmfile)), trim(adjustl(levname)), zi )
  call HistoryGet( trim(adjustl(atmfile)), trim(adjustl(tname)), ti )

!-- Checking reverse axis

  if(lon1(1)>lon1(2))then
     lonrev_flag=.true.
     call reverse_axis_r( lon1 )
  else
     lonrev_flag=.false.
  end if

  if(lat1(1)>lat1(2))then
     latrev_flag=.true.
     call reverse_axis_r( lat1 )
  else
     latrev_flag=.false.
  end if

  if(zi(1)<zi(2))then
     levrev_flag=.true.
     call reverse_axis_i( zi )
  else
     levrev_flag=.false.
  end if

  if(len_trim(adjustl(terfile))/=0)then
     call HistoryGet( trim(adjustl(terfile)), trim(adjustl(lontname)), lont )
     call HistoryGet( trim(adjustl(terfile)), trim(adjustl(lattname)), latt )

  !-- Checking reverse axis

     if(lont(1)>lont(2))then
        lontrev_flag=.true.
        call reverse_axis_r( lont )
     else
        lontrev_flag=.false.
     end if

     if(latt(1)>latt(2))then
        lattrev_flag=.true.
        call reverse_axis_r( latt )
     else
        lattrev_flag=.false.
     end if

  end if

  if(len_trim(adjustl(surfile))/=0)then
     call HistoryGet( trim(adjustl(surfile)), trim(adjustl(lonsname)), lons )
     call HistoryGet( trim(adjustl(surfile)), trim(adjustl(latsname)), lats )

  !-- Checking reverse axis

     if(lons(1)>lons(2))then
        lonsrev_flag=.true.
        call reverse_axis_r( lons )
     else
        lonsrev_flag=.false.
     end if

     if(lats(1)>lats(2))then
        latsrev_flag=.true.
        call reverse_axis_r( lats )
     else
        latsrev_flag=.false.
     end if

  end if

  if(pflag.eqv..true.)then
     pfact=100.0
  else
     pfact=1.0
  end if

  if(gphflag.eqv..true.)then
     gfact=1.0/g
  else
     gfact=1.0
  end if

  do i=1,nz
     z(i)=real(zi(i))*pfact
  end do

!-- setting each variable

  select case (ipflag)
  case (1)
     lon=lon1
     lat=lat1
  case (3)
     lon=lons
     lat=lats
  case (4)
     lon=lont
     lat=latt
  end select

  call time_zone_convert( ti(1), stime, etime )
  stime=etime
  ti(2:nt)=ti(2:nt)-ti(1)
  ti(1)=0

!-- reading terrain data

  if(trim(terfile)/='')then
     forma=trim(adjustl(lontname))//'=^1:^'  &
  &      //trim(adjustl(i2c_convert(nxt)))//','  &
  &      //trim(adjustl(lattname))//'=^1:^'  &
  &      //trim(adjustl(i2c_convert(nyt)))//','  &
  &      //trim(adjustl(tttname))//'=^1'

     call HistoryGet( trim(adjustl(terfile)), trim(adjustl(tername)),  &
  &                   tmpt, range=trim(adjustl(forma)) )

     call HistoryGetAttr( trim(adjustl(terfile)), trim(adjustl(tername)),  &
  &                       trim(adjustl(factname)), factor )
     call HistoryGetAttr( trim(adjustl(terfile)), trim(adjustl(tername)),  &
  &                       trim(adjustl(addname)), addoff )
     call HistoryGetAttr( trim(adjustl(terfile)), trim(adjustl(tername)),  &
  &                       trim(adjustl(undefname)), undef )

     call convert_short2float_2d( tmpt, tmptr, offset=addoff,  &
  &                               factor=factor, undef=undef,  &
  &                               undefc=undefc )

     !-- terrain data is set in reverse.
     call full_reverse_axis_2dr( lontrev_flag, lattrev_flag, tmptr )

     if(ipflag==4)then
        ter(1:nx,1:ny)=tmptr(1:nx,1:ny)
     else
        call auto_interpolation_2d( lont, latt, lon, lat, tmptr,  &
!        call auto_interpolation_2d( lont, lattr, lon, lat, tmptr,  &
  &                                 ter(1:nx,1:ny), undef=0.0, undefr=0.0,  &
  &                                 stdopt=.true. )
     end if

     call HistoryGet( trim(adjustl(terfile)), trim(adjustl(landname)),  &
  &                   tmpt, range=trim(adjustl(forma)) )

     call HistoryGetAttr( trim(adjustl(terfile)), trim(adjustl(landname)),  &
  &                       trim(adjustl(factname)), factor )
     call HistoryGetAttr( trim(adjustl(terfile)), trim(adjustl(landname)),  &
  &                       trim(adjustl(addname)), addoff )
     call HistoryGetAttr( trim(adjustl(terfile)), trim(adjustl(landname)),  &
  &                       trim(adjustl(undefname)), undef )

     call convert_short2float_2d( tmpt, tmptr, offset=addoff,  &
  &                               factor=factor, undef=undef,  &
  &                               undefc=undefc )

     !-- land mask data is set in reverse.
     call full_reverse_axis_2dr( lontrev_flag, lattrev_flag, tmptr )

     if(ipflag==4)then
        land(1:nx,1:ny)=tmptr(1:nx,1:ny)
     else
        call auto_interpolation_2d( lont, latt, lon, lat, tmptr,  &
!        call auto_interpolation_2d( lont, lattr, lon, lat, tmptr,  &
  &                                 land(1:nx,1:ny), undef=0.0, undefr=0.0,  &
  &                                 stdopt=.true. )
     end if

     !-- modifying terrain height
     do j=1,ny
        do i=1,nx
           if(nint(land(i,j))==land_flag(1))then   ! for ocean
              ter(i,j)=0.0
           end if
        end do
     end do

  end if

!-- count dord

  nc=8
  if(len_trim(adjustl(surfile))/=0)then
     ns=8
  else
     ns=0
  end if

!-- reading data

  do id=1,nt
     !-- reading surface part
     if(ns/=0)then
        do jd=1,ns
           if(len_trim(adjustl(ssvname(jd)))/=0)then
              forma=trim(adjustl(lonsname))//'=^1:^'  &
  &               //trim(adjustl(i2c_convert(nxs)))//','  &
  &               //trim(adjustl(latsname))//'=^1:^'  &
  &               //trim(adjustl(i2c_convert(nys)))//','  &
  &               //trim(adjustl(tstname))//'=^'  &
  &               //trim(adjustl(i2c_convert(id)))

              call HistoryGet( trim(adjustl(surfile)),  &
  &                            trim(adjustl(ssvname(jd))),  &
  &                            tmps1, range=trim(adjustl(forma)) )

              call HistoryGetAttr( trim(adjustl(surfile)),  &
  &                                trim(adjustl(ssvname(jd))),  &
  &                                trim(adjustl(factname)), factor )
              call HistoryGetAttr( trim(adjustl(surfile)),  &
  &                                trim(adjustl(ssvname(jd))),  &
  &                                trim(adjustl(addname)), addoff )
              call HistoryGetAttr( trim(adjustl(surfile)),  &
  &                                trim(adjustl(ssvname(jd))),  &
  &                                trim(adjustl(undefname)), undef )

              call convert_short2float_2d( tmps1, tmps2, offset=addoff,  &
  &                                        factor=factor, undef=undef,  &
  &                                        undefc=undefc )

              !-- 3d data is set in reverse.
              call full_reverse_axis_2dr( lonsrev_flag, latsrev_flag, tmps2 )

              if(ipflag==3)then
                 val2(1:nx,1:ny,jd)=tmps2(1:nx,1:ny)
              else
                 call auto_interpolation_2d( lons, lats, lon, lat, tmps2,  &
  &                                          val2(1:nx,1:ny,jd), stdopt=.true. )
              end if
           end if
        end do
     end if

     do jd=1,nc

        if(len_trim(adjustl(vname(jd)))/=0)then

           forma=trim(adjustl(lonname))//'=^1:^'  &
  &            //trim(adjustl(i2c_convert(nx1)))//','  &
  &            //trim(adjustl(latname))//'=^1:^'  &
  &            //trim(adjustl(i2c_convert(ny1)))//','  &
  &            //trim(adjustl(levname))//'=^1:^'  &
  &            //trim(adjustl(i2c_convert(nz)))//','  &
  &            //trim(adjustl(tname))//'=^'  &
  &            //trim(adjustl(i2c_convert(id)))

           call HistoryGet( trim(adjustl(atmfile)),  &
  &                         trim(adjustl(vname(jd))),  &
  &                         tmp1, range=trim(adjustl(forma)) )

           call HistoryGetAttr( trim(adjustl(atmfile)),  &
  &                             trim(adjustl(vname(jd))),  &
  &                             trim(adjustl(factname)), factor )
           call HistoryGetAttr( trim(adjustl(atmfile)),  &
  &                             trim(adjustl(vname(jd))),  &
  &                             trim(adjustl(addname)), addoff )
           call HistoryGetAttr( trim(adjustl(atmfile)),  &
  &                             trim(adjustl(vname(jd))),  &
  &                             trim(adjustl(undefname)), undef )

           call convert_short2float( tmp1, tmp2, offset=addoff,  &
  &                                  factor=factor, undef=undef,  &
  &                                  undefc=undefc )

           !-- 3d data is set in reverse.
           call full_reverse_axis_3dr( lonrev_flag, latrev_flag,  &
  &                                    levrev_flag, tmp2 )

           if(ipflag==1)then
              val3(1:nx,1:ny,1:nz,jd)=tmp2(1:nx,1:ny,1:nz)
           else
              do k=1,nz
                 call auto_interpolation_2d( lon1, lat1, lon, lat,  &
  &                                          tmp2(1:nx,1:ny,k),  &
  &                                          val3(1:nx,1:ny,k,jd),  &
  &                                          stdopt=.true. )
              end do
           end if

        end if

     end do

!-- inserting pressure value

     do k=1,nz
        val3(1:nx,1:ny,k,4)=z(k)
     end do

     do k=1,nz
        do j=1,ny
           do i=1,nx
              if(val3(i,j,k,1)/=undefc)then
                 val3(i,j,k,1)=val3(i,j,k,1)*gfact
              end if
           end do
        end do
     end do

!-- converting surface TD to RH

     if(ns/=0)then
        do j=1,ny
           do i=1,nx
              if(val2(i,j,7)/=undefc)then
                 val2(i,j,7)=TTd_2_RH_Bolton( val2(i,j,5), val2(i,j,7) )
              end if
!              if(val2(i,j,4)/=undefc)then
!                 val2(i,j,4)=val2(i,j,4)*pfact
!              end if
              if(ter(i,j)/=undefc)then
                 ter(i,j)=ter(i,j)*gfact
              end if
           end do
        end do
     end if

!-- output CReSS data
     call time_zone_convert( ti(id), stime, etime )
     oname='data.gpv'//trim(adjustl(i2c_convert( etime%year_d, forma='(i4.4)' )))  &
  &      //trim(adjustl(i2c_convert( etime%month_d, forma='(i2.2)' )))  &
  &      //trim(adjustl(i2c_convert( etime%day_d, forma='(i2.2)' )))  &
  &      //trim(adjustl(i2c_convert( etime%hour_d, forma='(i2.2)' )))  &
  &      //trim(adjustl(i2c_convert( etime%min_d, forma='(i2.2)' )))  &
  &      //'.bin'
     ct=1

     if(ns/=0)then   ! including surface data
        call write_file( trim(oname), nx, ny, ct, ter(1:nx,1:ny),  &
  &                      mode='replace' )
        ct=ct+1
        call write_file_3d( trim(oname), nx, ny, nz, ct, val3(1:nx,1:ny,1:nz,1) )
        ct=ct+nz
     else
        call write_file_3d( trim(oname), nx, ny, nz, ct,  &
  &                         val3(1:nx,1:ny,1:nz,1), mode='replace' )
        ct=ct+nz
     end if

     do jd=2,nc

        if(ns/=0)then
           if((len_trim(adjustl(ssvname(jd)))/=0).or.jd==4)then
              if(jd==7)then
                 if(rhflag.eqv..true.)then
                    call check_maxmin( 0.0e0, 100.0e0, val2(1:nx,1:ny,7:7), dl=1 )
                 else
                    call check_maxmin( 0.0e0, 1.0e0, val2(1:nx,1:ny,7:7), dl=1 )
                 end if
              end if

              call write_file( trim(oname), nx, ny, ct, val2(1:nx,1:ny,jd) )

              ct=ct+1
           else
              if((len_trim(adjustl(vname(jd)))/=0))then
                 call write_file( trim(oname), nx, ny, ct, vanish2d(1:nx,1:ny) )

                 ct=ct+1
              end if
           end if
        end if

        if((len_trim(adjustl(vname(jd)))/=0).or.jd==4)then
           if(jd==7)then
              if(rhflag.eqv..true.)then
                 call check_maxmin( 0.0e0, 100.0e0, val3(1:nx,1:ny,1:nz,jd), dl=1 )
              else
                 call check_maxmin( 0.0e0, 1.0e0, val3(1:nx,1:ny,1:nz,jd), dl=1 )
              end if
           end if
           call write_file_3d( trim(oname), nx, ny, nz, ct,  &
  &                            val3(1:nx,1:ny,1:nz,jd) )
           ct=ct+nz
        end if
     end do

     write(*,*) "Writing : ", trim(oname)

  end do

  write(*,*) "FInished normally."

contains

subroutine convert_short2float( ival, oval, factor, offset, undef, undefc )

  implicit none

  integer, intent(in) :: ival(:,:,:)
  real, intent(inout) :: oval(size(ival,1),size(ival,2),size(ival,3))
  double precision, intent(in) :: factor
  double precision, intent(in) :: offset
  integer, intent(in) :: undef
  real, intent(in) :: undefc
  integer :: i, j, k, ni, nj, nk

  ni=size(ival,1)
  nj=size(ival,2)
  nk=size(ival,3)

!$omp parallel default(shared)
!$omp do schedule(runtime) private(i,j,k)

  do k=1,nk
     do j=1,nj
        do i=1,ni
           if(ival(i,j,k)==undef)then
              oval(i,j,k)=undefc
           else
              oval(i,j,k)=real( dble(ival(i,j,k))*factor+offset )
           end if
        end do
     end do
  end do

!$omp end do
!$omp end parallel

end subroutine convert_short2float


subroutine convert_short2float_2d( ival, oval, factor, offset, undef, undefc )

  implicit none

  integer, intent(in) :: ival(:,:)
  real, intent(inout) :: oval(size(ival,1),size(ival,2))
  double precision, intent(in) :: factor
  double precision, intent(in) :: offset
  integer, intent(in) :: undef
  real, intent(in) :: undefc
  integer :: i, j, ni, nj

  ni=size(ival,1)
  nj=size(ival,2)

!$omp parallel default(shared)
!$omp do schedule(runtime) private(i,j)

  do j=1,nj
     do i=1,ni
        if(ival(i,j)==undef)then
           oval(i,j)=undefc
        else
           oval(i,j)=real( dble(ival(i,j))*factor+offset )
        end if
     end do
  end do

!$omp end do
!$omp end parallel

end subroutine convert_short2float_2d


subroutine check_maxmin( minv, maxv, val, dl )
  use max_min

  implicit none

  real, intent(in) :: minv
  real, intent(in) :: maxv
  real, intent(inout) :: val(:,:,:)
  integer, intent(in), optional :: dl  ! 1 = Not output, 2 = output for standard.
                                       ! default = 2
  integer :: i, j, k, nx, ny, nz, debug

  nx=size(val,1)
  ny=size(val,2)
  nz=size(val,3)

  if(present(dl))then
     debug=dl
  else
     debug=2
  end if

  do k=1,nz
     do j=1,ny
        do i=1,nx
           if(val(i,j,k)>maxv)then
              if(debug==2)then
                 write(*,*) "*** WARNING (check_maxmin) ***"
                 write(*,'(a28,1PE15.8,3i4)') "Detect value over max value.",  &
  &                                           val(i,j,k), i, j, k
                 write(*,'(a30,1PE15.8,a2,1PE15.8,a1)') "Forced the value to max value.",  &
  &                                                     val(i,j,k), '->', maxv, '.'
              end if
              val(i,j,k)=maxv
           end if
           if(val(i,j,k)<minv)then
              if(debug==2)then
                 write(*,*) "*** WARNING (check_maxmin) ***"
                 write(*,'(a28,1PE15.8,3i4)') "Detect value over min value.",  &
  &                                           val(i,j,k), i, j, k
                 write(*,'(a30,1PE15.8,a2,1PE15.8,a1)') "Forced the value to min value.",  &
  &                                                     val(i,j,k), '->', minv, '.'
              end if
              val(i,j,k)=minv
           end if
        end do
     end do
  end do

end subroutine check_maxmin


subroutine reverse_axis_i( val )

  implicit none

  integer, intent(inout) :: val(:)

  integer :: i, ni
  integer :: tmpr(size(val))

  ni=size(val)

  do i=1,ni
     tmpr(i)=val(ni-i+1)
  end do

  val=tmpr

end subroutine reverse_axis_i


subroutine reverse_axis_r( val )

  implicit none

  real, intent(inout) :: val(:)

  integer :: i, ni
  real :: tmpr(size(val))

  ni=size(val)

  do i=1,ni
     tmpr(i)=val(ni-i+1)
  end do

  val=tmpr

end subroutine reverse_axis_r


subroutine full_reverse_axis_2dr( flag1, flag2, val )

  implicit none

  logical, intent(in) :: flag1
  logical, intent(in) :: flag2
  real, intent(inout) :: val(:,:)

  integer :: i, j, ni, nj
  real :: tmpr(size(val,1),size(val,2))

  ni=size(val,1)
  nj=size(val,2)

  if(flag1.eqv..true.)then
     do j=1,nj
        call reverse_axis_r( val(:,j) )
     end do
  end if

  if(flag2.eqv..true.)then
     do i=1,ni
        call reverse_axis_r( val(i,:) )
     end do
  end if

end subroutine full_reverse_axis_2dr


subroutine full_reverse_axis_3dr( flag1, flag2, flag3, val )

  implicit none

  logical, intent(in) :: flag1
  logical, intent(in) :: flag2
  logical, intent(in) :: flag3
  real, intent(inout) :: val(:,:,:)

  integer :: i, j, k, ni, nj, nk
  real :: tmpr(size(val,1),size(val,2))

  ni=size(val,1)
  nj=size(val,2)
  nk=size(val,3)

  if(flag1.eqv..true.)then
!$omp parallel default(shared)
!$omp do schedule(runtime) private(j,k)
     do k=1,nk
        do j=1,nj
           call reverse_axis_r( val(:,j,k) )
        end do
     end do
!$omp end do
!$omp end parallel
  end if

  if(flag2.eqv..true.)then
!$omp parallel default(shared)
!$omp do schedule(runtime) private(j,k)
     do k=1,nk
        do i=1,ni
           call reverse_axis_r( val(i,:,k) )
        end do
     end do
!$omp end do
!$omp end parallel
  end if

  if(flag3.eqv..true.)then
!$omp parallel default(shared)
!$omp do schedule(runtime) private(j,k)
     do j=1,nj
        do i=1,ni
           call reverse_axis_r( val(i,j,:) )
        end do
     end do
!$omp end do
!$omp end parallel
  end if

end subroutine full_reverse_axis_3dr


end program NCEP_convert
