######################################################
# Author : Satoki Tsujino (satoki_at_gfd-dennou.org) #
######################################################
#!/usr/bin/ruby
require 'date'

# USAGE  : 
#  $ ruby auto_convert_GSM_MSM.rb

###### Edit in depending on user. 
datype = 'MSM'  # 'GSM' or 'MSM'
sfcopt = 'sfc'  # 'sfc' or 'nosfc'
stime = '20120825'  # initial time YYYYMMDD
etime = '20120826'  # ending time YYYYMMDD
wgrib_dir = '/home/satoki/Download/grib2/wgrib2/'   # path of wgrib2
###### Edit in depending on user. 

###### Don't edit the followings.
#-- Some settings.
rishurl = 'http://database.rish.kyoto-u.ac.jp/arch/jmadata/data/gpv/original/'
d3head = 'Z__C_RJTD_'
d2head = 'Z__C_RJTD_'
gpvhead = 'data.gpv'
gpvfoot = '00.bin'

if datype == 'GSM' then
   dt = 6
   d3foot = '0000_' + datype + '_GPV_Rgl_FD0000_grib2.bin'
   d3list = 'gsm_upper_list.dat'
   if sfcopt == 'sfc' then
      d2foot = '0000_' + datype + '_GPV_Rgl_FD0000_grib2.bin'
      d2list = 'gsm_sfc_list.dat'
   end
elsif datype == 'MSM' then
   dt = 3
   d3foot = '0000_' + datype + '_GPV_Rjp_L-pall_FH00-15_grib2.bin'
   d3list = 'msm_upper_list.dat'
   if sfcopt == 'sfc' then
      d2foot = '0000_' + datype + '_GPV_Rjp_Lsurf_FH00-15_grib2.bin'
      d2list = 'msm_sfc_list.dat'
   end
end

syear = stime[0..3].to_i
smonth = stime[4..5].to_i
sday = stime[6..7].to_i
eyear = etime[0..3].to_i
emonth = etime[4..5].to_i
eday = etime[6..7].to_i

sttime = Date.new( syear, smonth, sday )
entime = Date.new( eyear, emonth, eday )

dday = entime - sttime + 1
fnumber = dday.to_i * (24/dt)

confname = 'gpv_convert_' + datype + '_' + sfcopt + '.nml'

#-- Preparing file names.

orig3name = Array.new(fnumber)
orig2name = Array.new(fnumber)
wgrib3name = Array.new(fnumber)
wgrib2name = Array.new(fnumber)
dataname = Array.new(fnumber)
gpvname = Array.new(fnumber)
fulldate = Array.new(fnumber)
daydate = ''
hour = 0

for i in 0..fnumber-1
   daydate = sttime + (i*dt+hour)/24
   fulldate[i] = daydate.year.to_s.rjust(4,'0') + daydate.month.to_s.rjust(2,'0') + daydate.day.to_s.rjust(2,'0') + ((i*dt+hour)%24).to_s.rjust(2,'0')
   orig3name[i] = d3head + fulldate[i] + d3foot
   wgrib3name[i] = orig3name[i] + '.out'
   dataname[i] = orig3name[i] + '.out.out'
   gpvname[i] = gpvhead + fulldate[i] + gpvfoot
end

#-- Downloading original data
tmpdate = ''

for i in 0..fnumber-1
   tmpdate = fulldate[i]
   system("wget #{rishurl}#{tmpdate[0..3]}/#{tmpdate[4..5]}/#{tmpdate[6..7]}/#{orig3name[i]}")
end

#-- Downloading surface data

if sfcopt == 'sfc' then
   for i in 0..fnumber-1
      orig2name[i] = d2head + fulldate[i] + d2foot
      wgrib2name[i] = orig2name[i] + '.out'
   end
   for i in 0..fnumber-1
      tmpdate = fulldate[i]
      system("wget #{rishurl}#{tmpdate[0..3]}/#{tmpdate[4..5]}/#{tmpdate[6..7]}/#{orig2name[i]}")
   end
end

#-- Converting grib2 to grads
for i in 0..fnumber-1
   system("#{wgrib_dir}/wgrib2 #{orig3name[i]} | grep 'anl' | #{wgrib_dir}/wgrib2 -i #{orig3name[i]} -no_header -ieee #{wgrib3name[i]}")
end

if sfcopt == 'sfc' then
   for i in 0..fnumber-1
      system("#{wgrib_dir}/wgrib2 #{orig2name[i]} | grep 'anl' | #{wgrib_dir}/wgrib2 -i #{orig2name[i]} -no_header -ieee #{wgrib2name[i]}")
   end
end

#-- Converting grads to CReSS data
tmplist = ''

for i in 0..fnumber-1
   tmplist = tmplist + wgrib3name[i] + "\n"
end

f=open(d3list,'w')
f.write(tmplist)
f.close
tmplist = ''

if sfcopt == 'sfc' then
   for i in 0..fnumber-1
      tmplist = tmplist + wgrib2name[i] + "\n"
   end
   f=open(d2list,'w')
   f.write(tmplist)
   f.close
end

system("ulimit -s unlimited; ./gpv_convert < #{confname}")

#-- Rename CReSS data.

for i in 0..fnumber-1
   system("mv #{dataname[i]} #{gpvname[i]}")
   print dataname[i], ' -> ', gpvname[i], "\n"
end
