!---------------------------------------------------------------!
! This program is to convert wgrib2 data to CReSS data.         !
!                                                               !
! developed : Satoki Tsujino                                    !
!---------------------------------------------------------------!

program gpv_convert

  use file_operate
  use Statistics
  use Basis

  implicit none

  integer, parameter :: nzlim=100

!-- internal variables

  integer :: i, j, k, id, jd, ct, ir
  integer :: nx, ny, nl, nc, ns
  integer, dimension(7) :: counter

  real :: dlon, dlat, dlon1, dlat1, dlon2, dlat2, dlons, dlats
  real :: lonmin, latmin, lonmax, latmax
  real, allocatable, dimension(:) :: lon, lat, lon1, lat1, lon2, lat2, lont
  real, allocatable, dimension(:) :: lons, lats, latt
  real, allocatable, dimension(:,:) :: tmp1, tmp2, tmpt, tmptr, tmps, ter
  real, allocatable, dimension(:,:,:) :: val2, vals
  real, allocatable, dimension(:,:,:,:) :: val3
  character(1000) :: oname
  character(1000), allocatable, dimension(:) :: ulist, slist
  logical :: rhflag   ! moisture is RH (.true.) or mixing ratio (.false.).

!-- namelist variables

  integer :: nx1, ny1, nx2, ny2, nxt, nyt, nxs, nys, nz
  integer :: ipflag, iplev
  real :: lonmin1, latmin1, lonmin2, latmin2, lonmint, latmint
  real :: lonmax1, latmax1, lonmax2, latmax2, dlont, dlatt
  real :: lonmins, latmins, lonmaxs, latmaxs
  character(1) :: uflag, vflag, wflag, pflag, tflag, qflag, zflag, sflag
  character(1000) :: dord, d2ord, ofoot, rlist, terfile, surfile
  real :: z(nzlim)

  namelist /basic /uflag, vflag, wflag, pflag, tflag, qflag, zflag, sflag,  &
  &                dord, nx1, ny1, nx2, ny2, nz,  &
  &                lonmin1, latmin1, lonmin2, latmin2,  &
  &                lonmax1, latmax1, lonmax2, latmax2,  &
  &                ipflag, iplev, rhflag, ofoot, rlist
  namelist /vercoord /z
  namelist /terrain /terfile, nxt, nyt, lonmint, latmint, dlont, dlatt
  namelist /surface /surfile, d2ord, nxs, nys, lonmins, latmins, lonmaxs, latmaxs

  read(5,nml=basic)
  read(5,nml=vercoord)
  read(5,nml=terrain)
  read(5,nml=surface)

  if(nzlim<nz)then
     write(*,*) "*** ERROR (main) *** : nzlim = 100, but nz exceeds nzlim."
     stop
  end if

!-- setting each variable

  select case (ipflag)
  case (1)
     nx=nx1
     ny=ny1
     lonmin=lonmin1
     latmin=latmin1
     lonmax=lonmax1
     latmax=latmax1
  case (2)
     nx=nx2
     ny=ny2
     lonmin=lonmin2
     latmin=latmin2
     lonmax=lonmax2
     latmax=latmax2
  case (3)
     nx=nxs
     ny=nys
     lonmin=lonmins
     latmin=latmins
     lonmax=lonmaxs
     latmax=latmaxs
  case (4)
     nx=nxt
     ny=nyt
     lonmin=lonmint
     latmin=latmint
     lonmax=lonmint+dlont*real(nxt-1)
     latmax=latmint+dlatt*real(nyt-1)
  end select

!-- allocating some variables

  allocate(lon(nx))
  allocate(lat(ny))
  allocate(lon1(nx1))
  allocate(lat1(ny1))
  allocate(lon2(nx2))
  allocate(lat2(ny2))
  allocate(lons(nxs))
  allocate(lats(nys))
  allocate(lont(nxt))
  allocate(latt(nyt))

  allocate(tmp1(nx1,ny1))
  allocate(tmp2(nx2,ny2))
  allocate(tmps(nxs,nys))
  allocate(tmpt(nxt,nyt))
  allocate(tmptr(nxt,nyt))
  allocate(vals(nx,ny,len_trim(d2ord)))
  allocate(val2(nx,ny,len_trim(dord)))
  allocate(val3(nx,ny,nz,7))
  allocate(ter(nx,ny))

!-- setting some coord

  dlon=(lonmax-lonmin)/real(nx-1)
  dlat=(latmax-latmin)/real(ny-1)
  dlon1=(lonmax1-lonmin1)/real(nx1-1)
  dlat1=(latmax1-latmin1)/real(ny1-1)
  dlon2=(lonmax2-lonmin2)/real(nx2-1)
  dlat2=(latmax2-latmin2)/real(ny2-1)
  dlons=(lonmaxs-lonmins)/real(nxs-1)
  dlats=(latmaxs-latmins)/real(nys-1)

  lon=(/((lonmin+dlon*real(i-1)),i=1,nx)/)
  lat=(/((latmin+dlat*real(i-1)),i=1,ny)/)
  lon1=(/((lonmin1+dlon1*real(i-1)),i=1,nx1)/)
  lat1=(/((latmin1+dlat1*real(i-1)),i=1,ny1)/)
  lon2=(/((lonmin2+dlon2*real(i-1)),i=1,nx2)/)
  lat2=(/((latmin2+dlat2*real(i-1)),i=1,ny2)/)
  lons=(/((lonmins+dlons*real(i-1)),i=1,nxs)/)
  lats=(/((latmins+dlats*real(i-1)),i=1,nys)/)
  lont=(/((lonmint+dlont*real(i-1)),i=1,nxt)/)
  latt=(/((latmint+dlatt*real(i-1)),i=1,nyt)/)

!-- reading terrain data

  if(trim(terfile)/='')then
     call read_file( trim(terfile), nxt, nyt, 1, tmpt )

     !-- terrain data is set in reverse.
     do j=1,nyt
        do i=1,nxt
           tmptr(i,nyt-j+1)=tmpt(i,j)
        end do
     end do

     if(ipflag==4)then
        ter(1:nx,1:ny)=tmptr(1:nx,1:ny)
     else
        call auto_interpolation_2d( lont, latt, lon, lat, tmptr,  &
  &                                 ter(1:nx,1:ny), stdopt=.true. )
     end if
  end if

!-- reading list file

  nl=line_number_counter( trim(rlist) )
  allocate(ulist(nl))

  call read_file_text( trim(rlist), 1, nl, ulist )

  if(trim(surfile)/='')then
     allocate(slist(nl))
     call read_file_text( trim(surfile), 1, nl, slist )
  end if

!-- count dord

  nc=len_trim(dord)
  if(trim(surfile)/='')then
     ns=len_trim(d2ord)
  else
     ns=0
  end if

!-- reading data

  do id=1,nl
     !-- reading surface part
     if(ns/=0)then
        do jd=1,ns
           if(d2ord(jd:jd)/='x')then
              call read_file( trim(slist(id)), nxs, nys, jd, tmps(1:nxs,1:nys) )
              if(ipflag==3)then
                 vals(1:nx,1:ny,jd)=tmps(1:nx,1:ny)
              else
                 call auto_interpolation_2d( lons, lats, lon, lat, tmps,  &
  &                                          vals(1:nx,1:ny,jd), stdopt=.true. )
              end if
           end if
        end do
     end if

     do jd=1,nc

        if(dord(jd:jd)/='x')then
           if(jd<iplev)then
              call read_file( trim(ulist(id)), nx1, ny1, jd, tmp1(1:nx1,1:ny1) )

              if(ipflag==1)then
                 val2(1:nx,1:ny,jd)=tmp1(1:nx,1:ny)
              else
                 call auto_interpolation_2d( lon1, lat1, lon, lat, tmp1,  &
  &                                          val2(1:nx,1:ny,jd), stdopt=.true. )
              end if

           else

              call read_file( trim(ulist(id)), nx2, ny2, jd, tmp2(1:nx2,1:ny2),  &
  &                           offset=nx1*ny1*(iplev-1)+nx2*ny2*(jd-iplev) )

              if(ipflag==2)then
                 val2(1:nx,1:ny,jd)=tmp2(1:nx,1:ny)
              else
                 call auto_interpolation_2d( lon2, lat2, lon, lat, tmp2,  &
  &                                          val2(1:nx,1:ny,jd), stdopt=.true. )
              end if
           end if
        end if

     end do

!-- rearranging arrays

     counter=0
     do jd=1,nc
        if(dord(jd:jd)==zflag(1:1))then
           counter(1)=counter(1)+1
           val3(1:nx,1:ny,counter(1),1)=val2(1:nx,1:ny,jd)
        else if(dord(jd:jd)==uflag(1:1))then
           counter(2)=counter(2)+1
           val3(1:nx,1:ny,counter(2),2)=val2(1:nx,1:ny,jd)
        else if(dord(jd:jd)==vflag(1:1))then
           counter(3)=counter(3)+1
           val3(1:nx,1:ny,counter(3),3)=val2(1:nx,1:ny,jd)
        else if(dord(jd:jd)==pflag(1:1))then
           counter(4)=counter(4)+1
           val3(1:nx,1:ny,counter(4),4)=val2(1:nx,1:ny,jd)
        else if(dord(jd:jd)==tflag(1:1))then
           counter(5)=counter(5)+1
           val3(1:nx,1:ny,counter(5),5)=val2(1:nx,1:ny,jd)
        else if(dord(jd:jd)==wflag(1:1))then
           counter(6)=counter(6)+1
           val3(1:nx,1:ny,counter(6),6)=val2(1:nx,1:ny,jd)
        else if(dord(jd:jd)==qflag(1:1))then
           counter(7)=counter(7)+1
           val3(1:nx,1:ny,counter(7),7)=val2(1:nx,1:ny,jd)
        end if
     end do

!-- inserting zero value

     do jd=1,7
        if((counter(jd)==0).and.(jd/=6))then   ! only height and pressure
           do k=1,nz
              val3(1:nx,1:ny,k,jd)=z(k)
              counter(jd)=counter(jd)+1
           end do
        else if((counter(jd)<nz).and.(counter(jd)>0).and.(jd/=6))then   ! maybe water vapor
           do k=counter(jd)+1,nz
              val3(1:nx,1:ny,k,jd)=0.0
              counter(jd)=counter(jd)+1
           end do
        end if
     end do

!-- output CReSS data
     oname=trim(ulist(id))//trim(ofoot)
     ct=1

     if(ns/=0)then   ! including surface data
        call write_file( trim(oname), nx, ny, ct, ter(1:nx,1:ny),  &
  &                      mode='replace' )
        ct=ct+1
        call write_file_3d( trim(oname), nx, ny, nz, ct, val3(1:nx,1:ny,1:nz,1) )
        ct=ct+nz
     else
        call write_file( trim(oname), nx, ny, ct, val3(1:nx,1:ny,ct,1),  &
  &                      mode='replace' )
        ct=ct+1
        call write_file_3d( trim(oname), nx, ny, nz-1, ct, val3(1:nx,1:ny,2:nz,1) )
        ct=ct+nz-1
     end if

     do jd=2,7

        if(counter(jd)/=0)then
           if(ns/=0)then
              do k=1,ns
                 if(d2ord(k:k)==uflag(1:1).and.jd==2)then
                    ir=k
                    exit
                 else if(d2ord(k:k)==vflag(1:1).and.jd==3)then
                    ir=k
                    exit
                 else if(d2ord(k:k)==pflag(1:1).and.jd==4)then
                    ir=k
                    exit
                 else if(d2ord(k:k)==tflag(1:1).and.jd==5)then
                    ir=k
                    exit
                 else if(d2ord(k:k)==wflag(1:1).and.jd==6)then
                    ir=k
                    exit
                 else if(d2ord(k:k)==qflag(1:1).and.jd==7)then
                    ir=k
                    if(rhflag.eqv..true.)then
                       call check_maxmin( 0.0e0, 100.0e0, vals(1:nx,1:ny,ir:ir), dl=1 )
                    else
                       call check_maxmin( 0.0e0, 1.0e0, vals(1:nx,1:ny,ir:ir), dl=1 )
                    end if
                    exit
                 end if
              end do
              call write_file( trim(oname), nx, ny, ct, vals(1:nx,1:ny,ir) )
              ct=ct+1
           end if

           if(jd==7)then
              if(rhflag.eqv..true.)then
                 call check_maxmin( 0.0e0, 100.0e0, val3(1:nx,1:ny,1:nz,jd), dl=1 )
              else
                 call check_maxmin( 0.0e0, 1.0e0, val3(1:nx,1:ny,1:nz,jd), dl=1 )
              end if
           end if
           call write_file_3d( trim(oname), nx, ny, nz, ct, val3(1:nx,1:ny,1:nz,jd) )
           ct=ct+nz
        end if
     end do

     write(*,*) "Writing : ", trim(oname)

  end do

  write(*,*) "FInished normally."


contains

subroutine check_maxmin( minv, maxv, val, dl )
  use max_min

  implicit none

  real, intent(in) :: minv
  real, intent(in) :: maxv
  real, intent(inout) :: val(:,:,:)
  integer, intent(in), optional :: dl  ! 1 = Not output, 2 = output for standard.
                                       ! default = 2
  integer :: i, j, k, nx, ny, nz, debug

  nx=size(val,1)
  ny=size(val,2)
  nz=size(val,3)

  if(present(dl))then
     debug=dl
  else
     debug=2
  end if

  do k=1,nz
     do j=1,ny
        do i=1,nx
           if(val(i,j,k)>maxv)then
              if(debug==2)then
                 write(*,*) "*** WARNING (check_maxmin) ***"
                 write(*,'(a28,1PE15.8,3i4)') "Detect value over max value.",  &
  &                                           val(i,j,k), i, j, k
                 write(*,'(a30,1PE15.8,a2,1PE15.8,a1)') "Forced the value to max value.",  &
  &                                                     val(i,j,k), '->', maxv, '.'
              end if
              val(i,j,k)=maxv
           end if
           if(val(i,j,k)<minv)then
              if(debug==2)then
                 write(*,*) "*** WARNING (check_maxmin) ***"
                 write(*,'(a28,1PE15.8,3i4)') "Detect value over min value.",  &
  &                                           val(i,j,k), i, j, k
                 write(*,'(a30,1PE15.8,a2,1PE15.8,a1)') "Forced the value to min value.",  &
  &                                                     val(i,j,k), '->', minv, '.'
              end if
              val(i,j,k)=minv
           end if
        end do
     end do
  end do

end subroutine check_maxmin

end program gpv_convert
