!---------------------------------------------------------------!
! This program is to convert wgrib2 data to CReSS data.         !
!                                                               !
! developed : Satoki Tsujino                                    !
!---------------------------------------------------------------!

program convert_nc
  use gtool_history
  use Basis
  use File_Operate
  use Thermo_Const
  use Statistics
  use Phys_Const
  use Math_Const

  implicit none

!-- parameter
  integer, parameter :: cnum=6

!-- namelist values
  integer :: nx, ny, nz, nt
  integer :: nxs, nys, nts
  integer :: ht_ord
  integer, dimension(cnum) :: wflag, wsflag
  integer, dimension(4) :: axis_ord
  integer, dimension(2) :: axiss_ord
  character(20), dimension(cnum) :: ctype, cstype, typ_flag, typs_flag
  character(20), dimension(4) :: axis_name, axis_typ
  character(20), dimension(2) :: axiss_name, axiss_typ
  character(1000) :: fname, sfname, htfile
  character(4) :: interpo_flag
  logical :: sflag, yrev, vert_conv
  logical, dimension(cnum) :: off_flag, offs_flag
  logical, dimension(4) :: axis_off
  logical, dimension(2) :: axiss_off

  integer :: i, j, k, l, m, n, nl, counter
  integer, allocatable, dimension(:) :: time
  real, allocatable, dimension(:) :: lon, lat, vert
  real, allocatable, dimension(:) :: lons, lats
  real, allocatable, dimension(:,:) :: terrain, terrain3
  integer, allocatable, dimension(:,:,:) :: ivals
  integer, allocatable, dimension(:,:,:,:) :: ival
  real, allocatable, dimension(:,:,:) :: tmpvals
  double precision, allocatable, dimension(:,:,:) :: dvals
  double precision, allocatable, dimension(:,:,:,:) :: dval
  real, allocatable, dimension(:,:,:,:) :: ovals, convals
  real, allocatable, dimension(:,:,:,:,:) :: oval, conval

  character(100), allocatable, dimension(:) :: cval, csval
  character(12) :: ctime
  character(7) :: mode
  character(1000) :: outname

  namelist /upper /nx, ny, nz, nt, fname, ctype, wflag, off_flag,  &
  &                typ_flag, axis_name, axis_ord, axis_typ, axis_off
  namelist /surface /nxs, nys, nts, sfname, cstype, wsflag, offs_flag,  &
  &                  typs_flag, axiss_name, axiss_ord, axiss_typ, axiss_off,  &
  &                  htfile, ht_ord
  namelist /inter /yrev, vert_conv, sflag, interpo_flag
  read(5,nml=upper)
  read(5,nml=surface)
  read(5,nml=inter)

  allocate(lon(nx))
  allocate(lat(ny))
  allocate(vert(nz))
  allocate(time(nt))
  allocate(ival(nx,ny,nz,nt))
  allocate(dval(nx,ny,nz,nt))
  allocate(conval(nx,ny,nz,nt,cnum))

  allocate(lons(nxs))
  allocate(lats(nys))
  allocate(terrain(nxs,nys))
  allocate(ivals(nxs,nys,nts))
  allocate(dvals(nxs,nys,nts))
  allocate(tmpvals(nxs,nys,nts))
  allocate(convals(nxs,nys,nt,cnum))

  if(sflag.eqv..true.)then
     select case (trim(interpo_flag))
     case ('surf')
        allocate(oval(nxs,nys,nz,nt,cnum))
        allocate(ovals(nxs,nys,nt,cnum))
     case ('uppr')
        allocate(oval(nx,ny,nz,nt,cnum))
        allocate(ovals(nx,ny,nt,cnum))
        allocate(terrain3(nx,ny))
     end select
  else
     allocate(oval(nx,ny,nz,nt,cnum))
  end if

  nl=line_number_counter( trim(fname) )

  allocate(cval(nl))
  allocate(csval(nl))

  !-- reading file list (3d)
  call read_file_text( trim(fname), 1, nl, cval )

  !-- reading fixed values section (START)
  call HistoryGet( trim(cval(1)), trim(axis_name(1)), lon )
  call HistoryGet( trim(cval(1)), trim(axis_name(2)), lat )
  call HistoryGet( trim(cval(1)), trim(axis_name(3)), vert )

  if(vert_conv.eqv..true.)then
     do k=1,nz
        vert(k)=vert(k)*100.0
     end do
  end if

  !-- reading file list (2d)
  if(sflag.eqv..true.)then
     call read_file_text( trim(sfname), 1, nl, csval )

     !-- reading file list (terrain)
     call read_file( trim(htfile), nxs, nys, 1, terrain )

     !-- gtool history get (1d axis)
     call HistoryGet( trim(csval(1)), trim(axiss_name(1)), lons )
     call HistoryGet( trim(csval(1)), trim(axiss_name(2)), lats )

  end if

  !-- yrev option and interpolation terrain from surf to 3d
  if(yrev.eqv..true.)then
     call yreverse_1d(lat)

     if(sflag.eqv..true.)then
        call yreverse_1d(lats)
        call yreverse_2d(terrain)
     end if
  end if

  if(sflag.eqv..true.)then
     if(trim(interpo_flag)=='uppr')then
        call auto_interpolation_2d( lons, lats, lon, lat,  &
  &          terrain(1:nxs,1:nys), terrain3(1:nx,1:ny) )
     end if
  end if
  !-- reading fixed values section (END)

  !-- reading varying values section (START)
  do m=1,nl

     write(*,*) "Start reading ", trim(cval(m)), '.'

     !-- gtool history get (3d)

     do l=1,cnum
        select case (trim(typ_flag(l)))
        case ('i')
           call HistoryGet( trim(cval(m)), trim(ctype(l)),  &
  &                         ival(1:nx,1:ny,1:nz,1:nt) )

           if(off_flag(l).eqv..true.)then  ! offset is only short type.
              call calc_offset_4d( trim(cval(m)), trim(ctype(l)),  &
  &                                ival(1:nx,1:ny,1:nz,1:nt),  &
  &                                conval(1:nx,1:ny,1:nz,1:nt,l) )
           end if

        case ('r')
           call HistoryGet( trim(cval(m)), trim(ctype(l)),  &
  &                         conval(1:nx,1:ny,1:nz,1:nt,l) )

        case ('d')
           call HistoryGet( trim(cval(m)), trim(ctype(l)),  &
  &                         dval(1:nx,1:ny,1:nz,1:nt) )
           do n=1,nt
              do k=1,nz
                 do j=1,ny
                    do i=1,nx
                       conval(i,j,k,n,l)=real(dval(i,j,k,n))
                    end do
                 end do
              end do
           end do

        end select
     end do

     !-- gtool history get (1d axis)
     call HistoryGet( trim(cval(m)), trim(axis_name(4)), time )

     !-- gtool history get (2d)

     if(sflag.eqv..true.)then
        write(*,*) "Including surface data ", trim(csval(m)), '.'
        do l=1,cnum
           select case (trim(typs_flag(l)))
           case ('i')
              call HistoryGet( trim(csval(m)), trim(cstype(l)),  &
  &                            ivals(1:nxs,1:nys,1:nts) )

              if(offs_flag(l).eqv..true.)then
                 call calc_offset_3d( trim(csval(m)), trim(cstype(l)),  &
  &                                   ivals(1:nxs,1:nys,1:nts),  &
  &                                   tmpvals(1:nxs,1:nys,1:nts) )
                 do n=1,nt
                    do j=1,nys
                       do i=1,nxs
                          convals(i,j,n,l)=tmpvals(i,j,(n-1)*(nts/nt)+1)
                       end do
                    end do
                 end do
              end if

           case ('r')
              call HistoryGet( trim(csval(m)), trim(cstype(l)),  &
  &                            tmpvals(1:nxs,1:nys,1:nts) )
              do n=1,nt
                 do j=1,nys
                    do i=1,nxs
                       convals(i,j,n,l)=tmpvals(i,j,(n-1)*(nts/nt)+1)
                    end do
                 end do
              end do

           case ('d')
              call HistoryGet( trim(csval(m)), trim(cstype(l)),  &
  &                            dvals(1:nxs,1:nys,1:nts) )
              do n=1,nt
                 do j=1,nys
                    do i=1,nxs
                       convals(i,j,n,l)=real(dvals(i,j,(n-1)*(nts/nt)+1))
                    end do
                 end do
              end do

           end select
        end do
     end if

     !-- yrev option
     if(yrev.eqv..true.)then
        call yreverse_5d( conval )
        if(sflag.eqv..true.)then
           call yreverse_4d( convals )
        end if
     end if

     !-- interpolation (only two type data, surf and uppr)
     if(sflag.eqv..true.)then
        select case (trim(interpo_flag))
        case ('surf')
!$omp parallel default(shared)
!$omp do private(k,n,l)
           do l=1,cnum
              do n=1,nt
                 do k=1,nz
                    call auto_interpolation_2d( lon, lat, lons, lats,  &
  &                                             conval(1:nx,1:ny,k,n,l),  &
  &                                             oval(1:nxs,1:nys,k,n,l) )
                 end do
              end do
           end do
!$omp end do
!$omp end parallel

        case ('uppr')
!$omp parallel default(shared)
!$omp do private(n,l)
           do l=1,cnum
              do n=1,nt
                 call auto_interpolation_2d( lons, lats, lon, lat,  &
  &                   convals(1:nxs,1:nys,n,l),  &
  &                   ovals(1:nx,1:ny,n,l) )
              end do
           end do
!$omp end do
!$omp end parallel

        end select
     end if

  !-- sorting data from NetCDF to CReSS
     if(sflag.eqv..true.)then
        select case (trim(interpo_flag))
        case ('surf')
           call sort_CReSS( oval, wflag, convals, wsflag )
        case ('uppr')
           call sort_CReSS( conval, wflag, ovals, wsflag )
        end select
     else
        call sort_CReSS( conval, wflag )
     end if

  !-- writing data to GrADS format file
     do l=1,nt
        counter=1
        call output_time( trim(cval(m)), 'time', time(l), ctime )
        outname='data.gpv'//ctime//'.bin'

        !-- setting vertical axis to 3-dimensional array
        if(sflag.eqv..true.)then

           select case (trim(interpo_flag))
           case ('surf')
              do k=1,nz
                 oval(1:nxs,1:nys,k,l,axis_ord(3))=vert(k)
              end do
              convals(1:nxs,1:nys,l,ht_ord)=terrain(1:nxs,1:nys)
           case ('uppr')
              do k=1,nz
                 conval(1:nx,1:ny,k,l,axis_ord(3))=vert(k)
              end do
              ovals(1:nx,1:ny,l,ht_ord)=terrain3(1:nx,1:ny)
           end select

        else

           do k=1,nz
              conval(1:nx,1:ny,k,l,axis_ord(3))=vert(k)
           end do

        end if

        do k=1,cnum

           if(sflag.eqv..true.)then
              if(counter==1)then
                 mode='replace'
              else
                 mode='old    '
              end if

              select case (trim(interpo_flag))
              case ('surf')
                 call write_file( trim(outname), nxs, nys, counter,  &
  &                               convals(1:nxs,1:nys,l,k), mode=trim(mode) )
              case ('uppr')
                 call write_file( trim(outname), nx, ny, counter,  &
  &                               ovals(1:nx,1:ny,l,k), mode=trim(mode) )
              end select
              counter=counter+1
           end if

           if(counter==1)then
              mode='replace'
           else
              mode='old    '
           end if

           if(sflag.eqv..true.)then
              select case (trim(interpo_flag))
              case ('surf')
                 call write_file_3d( trim(outname), nxs, nys, nz, counter,  &
  &                   oval(1:nxs,1:nys,1:nz,l,k), mode=trim(mode) )
              case ('uppr')
                 call write_file_3d( trim(outname), nx, ny, nz, counter,  &
  &                   conval(1:nx,1:ny,1:nz,l,k), mode=trim(mode) )
              end select
           else
              call write_file_3d( trim(outname), nx, ny, nz, counter,  &
  &                conval(1:nx,1:ny,1:nz,l,k), mode=trim(mode) )
           end if

           counter=counter+nz
        end do

        write(*,*) "Finish producting ", trim(outname), '.'

     end do

  end do
  !-- reading varying values section (END)

  write(*,*) "Finish converting."

  !-- program finish.


contains     ! sub programs


subroutine calc_offset_2d( filename, val_name, val, conval, undef )
! NetCDF  Attribute 饪եåͤɤ߹, ʪ̤򸵤ͤľ.
  implicit none
  character(*), intent(in) :: filename        ! ɤ߹ NetCDF ե̾.
  character(*), intent(in) :: val_name        ! Ѵѿ̾.
  integer, dimension(:,:), intent(in) :: val     ! Ѵѿ.
  real, dimension(:,:), intent(inout) :: conval     ! Ѵѿ.
  real, optional :: undef         ! ̤.

  integer :: i, j
  integer :: nx, ny
  real :: coe, off
  character(100) :: att_name      ! °̾. (MSM ǤϷޤäƤ)
  logical :: undeflag

  if(present(undef))then
     undeflag=.true.
  else
     undeflag=.false.
  end if

  nx=size(val,1)
  ny=size(val,2)

  !-- reading attributes for the value

  att_name='scale_factor'
  call HistoryGetAttr( trim(filename), trim(val_name), trim(att_name), coe )

  att_name='add_offset'
  call HistoryGetAttr( trim(filename), trim(val_name), trim(att_name), off )

  if(undeflag.eqv..true.)then

     do j=1,ny
        do i=1,nx
           if(val(i,j)/=undef)then
              conval(i,j)=real(val(i,j))*coe+off
           end if
        end do
     end do

  else

     do j=1,ny
        do i=1,nx
           conval(i,j)=real(val(i,j))*coe+off
        end do
     end do

  end if

end subroutine calc_offset_2d


subroutine calc_offset_3d( filename, val_name, val, conval, undef )
! NetCDF  Attribute 饪եåͤɤ߹, ʪ̤򸵤ͤľ.
  implicit none
  character(*), intent(in) :: filename        ! ɤ߹ NetCDF ե̾.
  character(*), intent(in) :: val_name        ! Ѵѿ̾.
  integer, dimension(:,:,:), intent(in) :: val     ! Ѵѿ.
  real, dimension(:,:,:), intent(inout) :: conval     ! Ѵѿ.
  real, optional :: undef         ! ̤.

  integer :: i, j, k
  integer :: nx, ny, nz
  real :: coe, off
  character(100) :: att_name      ! °̾. (MSM ǤϷޤäƤ)
  logical :: undeflag

  if(present(undef))then
     undeflag=.true.
  else
     undeflag=.false.
  end if

  nx=size(val,1)
  ny=size(val,2)
  nz=size(val,3)

  !-- reading attributes for the value

  att_name='scale_factor'
  call HistoryGetAttr( trim(filename), trim(val_name), trim(att_name), coe )

  att_name='add_offset'
  call HistoryGetAttr( trim(filename), trim(val_name), trim(att_name), off )

  if(undeflag.eqv..true.)then

     do k=1,nz
        do j=1,ny
           do i=1,nx
              if(val(i,j,k)/=undef)then
                 conval(i,j,k)=real(val(i,j,k))*coe+off
              end if
           end do
        end do
     end do

  else

     do k=1,nz
        do j=1,ny
           do i=1,nx
              conval(i,j,k)=real(val(i,j,k))*coe+off
           end do
        end do
     end do

  end if


end subroutine calc_offset_3d


subroutine calc_offset_4d( filename, val_name, val, conval, undef )
! NetCDF  Attribute 饪եåͤɤ߹, ʪ̤򸵤ͤľ.
  implicit none
  character(*), intent(in) :: filename        ! ɤ߹ NetCDF ե̾.
  character(*), intent(in) :: val_name        ! Ѵѿ̾.
  integer, dimension(:,:,:,:), intent(in) :: val     ! Ѵѿ.
  real, dimension(:,:,:,:), intent(inout) :: conval     ! Ѵѿ.
  real, optional :: undef         ! ̤.

  integer :: i, j, k, l
  integer :: nx, ny, nz, nt
  real :: coe, off
  character(100) :: att_name      ! °̾. (MSM ǤϷޤäƤ)
  logical :: undeflag

  if(present(undef))then
     undeflag=.true.
  else
     undeflag=.false.
  end if

  nx=size(val,1)
  ny=size(val,2)
  nz=size(val,3)
  nt=size(val,4)

  !-- reading attributes for the value

  att_name='scale_factor'
  call HistoryGetAttr( trim(filename), trim(val_name), trim(att_name), coe )

  att_name='add_offset'
  call HistoryGetAttr( trim(filename), trim(val_name), trim(att_name), off )

  if(undeflag.eqv..true.)then

     do l=1,nt
        do k=1,nz
           do j=1,ny
              do i=1,nx
                 if(val(i,j,k,l)/=undef)then
                    conval(i,j,k,l)=real(val(i,j,k,l))*coe+off
                 end if
              end do
           end do
        end do
     end do

  else

     do l=1,nt
        do k=1,nz
           do j=1,ny
              do i=1,nx
                 conval(i,j,k,l)=real(val(i,j,k,l))*coe+off
              end do
           end do
        end do
     end do

  end if

end subroutine calc_offset_4d


subroutine sort_CReSS( val, wflag, vals, wsflag )
  implicit none
  real, dimension(:,:,:,:,:), intent(inout) :: val      ! 4 ǡ
  integer, dimension(size(val,5)), intent(in) :: wflag  ! Ǽľ.
  real, dimension(size(val,1),size(val,2),size(val,4),size(val,5)),  &
  &                 intent(inout), optional :: vals     ! 3 ǡ
  integer, dimension(size(val,5)), intent(in),  &
  &                                  optional :: wsflag ! Ǽľ.
  integer :: nx, ny, nz, nt, nc, i, j, k, l, m, n, q
  real, allocatable, dimension(:,:,:,:,:) :: tmpval
  real, allocatable, dimension(:,:,:,:) :: tmpvals

  nx=size(val,1)
  ny=size(val,2)
  nz=size(val,3)
  nt=size(val,4)
  nc=size(val,5)

  allocate(tmpval(nx,ny,nz,nt,nc))
  if(present(vals))then
     allocate(tmpvals(nx,ny,nt,nc))
  end if

  tmpval(1:nx,1:ny,1:nz,1:nt,1:nc)=val(1:nx,1:ny,1:nz,1:nt,1:nc)
  if(present(vals))then
     tmpvals(1:nx,1:ny,1:nt,1:nc)=vals(1:nx,1:ny,1:nt,1:nc)
  end if

  do q=1,nc
     if(wflag(q)/=0)then
        do l=1,nt
           do k=1,nz
              do j=1,ny
                 do i=1,nx
                    val(i,j,k,l,wflag(q))=tmpval(i,j,k,l,q)
                 end do
              end do
           end do
        end do
     end if
  end do

  if(present(vals))then
     do q=1,nc
        if(wflag(q)/=0)then
           do l=1,nt
              do j=1,ny
                 do i=1,nx
                    vals(i,j,l,wflag(q))=tmpvals(i,j,l,q)
                 end do
              end do
           end do
        end if
     end do
  end if

end subroutine sort_CReSS


subroutine output_time( fname, vname, time, ctime )
  implicit none
  character(*), intent(in) :: fname   ! ɤ߹ NetCDF ե̾.
  character(*), intent(in) :: vname   ! ɤ߹ѿ̾.
  integer, intent(in) :: time         !  [hour].
  character(*), intent(inout) :: ctime  ! 񤭽Ф.

  character(1000) :: tmpc
  character(2) :: tmptc
  integer :: tmpt

  call HistoryGetAttr( trim(fname), trim(vname), 'units', tmpc )

  tmpt=time+c2i_convert( tmpc(24:25) )
  tmptc=i2c_convert( tmpt, '(i2.2)' )
  ctime=tmpc(13:16)//tmpc(18:19)//tmpc(21:22)//tmptc(1:2)//'00'

end subroutine output_time


subroutine yreverse_1d( val )
  implicit none
  real, dimension(:), intent(inout) :: val
  real, dimension(size(val)) :: tmpval
  integer :: nx, i

  nx=size(val)

  do i=1,nx
     tmpval(nx-i+1)=val(i)
  end do

  do i=1,nx
     val(i)=tmpval(i)
  end do

end subroutine yreverse_1d


subroutine yreverse_2d( val )
  implicit none
  real, dimension(:,:), intent(inout) :: val
  real, dimension(size(val,1),size(val,2)) :: tmpval
  integer :: nx, ny, i, j

  nx=size(val,1)
  ny=size(val,2)

  do j=1,ny
     do i=1,nx
        tmpval(i,ny-j+1)=val(i,j)
     end do
  end do

  do j=1,ny
     do i=1,nx
        val(i,j)=tmpval(i,j)
     end do
  end do

end subroutine yreverse_2d


subroutine yreverse_4d( val )
  implicit none
  real, dimension(:,:,:,:), intent(inout) :: val
  real, dimension(size(val,1),size(val,2),size(val,3),size(val,4)) :: tmpval
  integer :: nx, ny, nz, nt, i, j, k, l

  nx=size(val,1)
  ny=size(val,2)
  nz=size(val,3)
  nt=size(val,4)

  do l=1,nt
     do k=1,nz
        do j=1,ny
           do i=1,nx
              tmpval(i,ny-j+1,k,l)=val(i,j,k,l)
           end do
        end do
     end do
  end do

  do l=1,nt
     do k=1,nz
        do j=1,ny
           do i=1,nx
              val(i,j,k,l)=tmpval(i,j,k,l)
           end do
        end do
     end do
  end do

end subroutine yreverse_4d


subroutine yreverse_5d( val )
  implicit none
  real, dimension(:,:,:,:,:), intent(inout) :: val
  real, dimension(size(val,1),size(val,2),size(val,3),size(val,4),size(val,5)) :: tmpval
  integer :: nx, ny, nz, nt, nl, i, j, k, l, m

  nx=size(val,1)
  ny=size(val,2)
  nz=size(val,3)
  nt=size(val,4)
  nl=size(val,5)

  do m=1,nl
     do l=1,nt
        do k=1,nz
           do j=1,ny
              do i=1,nx
                 tmpval(i,ny-j+1,k,l,m)=val(i,j,k,l,m)
              end do
           end do
        end do
     end do
  end do

  do m=1,nl
     do l=1,nt
        do k=1,nz
           do j=1,ny
              do i=1,nx
                 val(i,j,k,l,m)=tmpval(i,j,k,l,m)
              end do
           end do
        end do
     end do
  end do

end subroutine yreverse_5d


end program
