program read_mgdsst_nc
  ! FORA Υꥸʥǡ (NetCDF)  GrADS Ѵץ
  ! sst_list ˺ǡե񤭹, ưǺ.

  use Basis
  use Statistics
  use file_operate
  use gtool_history

  implicit none
!  integer, parameter :: nxd=301, nyd=251, nzd=17   ! used as 3-d temperature
  integer, parameter :: nxd=501, nyd=401, nzd=1  ! used as original cress sst
  real, parameter :: dlon=0.1  ! resolution of longtitude
  real, parameter :: dlat=0.1  ! resolution of latitude
  real, parameter :: lonmin=117.0  ! data start point (long)
  real, parameter :: latmin=15.0  ! data start point (lat)
  real, parameter :: undefc=-1.0e35
!  real, parameter :: sst_const=0.0  ! no conversion
  real, parameter :: sst_const=273.15  ! converting from degC to K
  character(80), parameter :: dat_name='sst_list'
  character(80), parameter :: vname='to'
  character(80), parameter :: xname='LONGITUDE'
  character(80), parameter :: yname='LATITUDE'
  character(80), parameter :: zname='DEPTH'
  character(80), parameter :: tname='TIME'
  character(100), parameter :: undefchar='missing_value'

  type(dtime) :: itime, otime
  integer :: nx, ny, nz, nxp, nyp, nzp
  real :: time(1)
  real :: undef
  real, pointer, dimension(:) :: a
  real :: lond(nxd), latd(nyd), zd(nzd), sstd(nxd,nyd,nzd)
  real, allocatable, dimension(:) :: lon, lat
  real, allocatable, dimension(:,:,:) :: sst
  character(12) :: date
  character(100) :: forma
  character(1000), allocatable, dimension(:) :: dat_list
  character(1000) :: oname
  integer :: dat_line
  integer :: i, j, k, l, dtype

  dat_line=line_number_counter( trim(dat_name) )
  allocate(dat_list(dat_line))
  call read_file_text( trim(dat_name), 1, dat_line, dat_list )

  write(*,*) "*** MESSAGE (main) *** : sst constant is ", sst_const

  nz=nzd
  nzp=nzd

  lond=(/((lonmin+dlon*real(i-1)),i=1,nxd)/)
  latd=(/((latmin+dlat*real(i-1)),i=1,nyd)/)
  itime%year_d=1970
  itime%month_d=1
  itime%day_d=1
  itime%hour_d=0
  itime%min_d=0
  itime%sec_d=0

  do i=1,dat_line

     nullify(a)
     call HistoryGetPointer( trim(adjustl(dat_list(i))),  &
  &                          trim(adjustl(xname)), a )
     nxp=size(a)

     nullify(a)
     call HistoryGetPointer( trim(adjustl(dat_list(i))),  &
  &                          trim(adjustl(yname)), a )
     nyp=size(a)

     if(i==1)then
        call HistoryGet( trim(adjustl(dat_list(i))), trim(adjustl(zname)),  &
  &                      zd(1:nzd), range=trim(adjustl(zname))//'=^1:^'//  &
  &                                       trim(adjustl(i2c_convert(nzd))) )

        nx=nxp
        ny=nyp

        allocate(lon(nx))
        allocate(lat(ny))
        allocate(sst(nx,ny,nz))
     else
        if(nx/=nxp.or.ny/=nyp)then
           deallocate(lon)
           deallocate(lat)
           deallocate(sst)

           nx=nxp
           ny=nyp

           allocate(lon(nx))
           allocate(lat(ny))
           allocate(sst(nx,ny,nz))
        end if
     end if

     forma=trim(adjustl(xname))//'=^1:^'//trim(adjustl(i2c_convert(nx)))//  &
  &     ','//trim(adjustl(yname))//'=^1:^'//trim(adjustl(i2c_convert(ny)))//  &
  &     ','//trim(adjustl(zname))//'=^1:^'//trim(adjustl(i2c_convert(nz)))//  &
  &     ',TIME=^1'

     call HistoryGet( trim(adjustl(dat_list(i))), trim(adjustl(tname)), time )
     call HistoryGet( trim(adjustl(dat_list(i))), trim(adjustl(xname)), lon )
     call HistoryGet( trim(adjustl(dat_list(i))), trim(adjustl(yname)), lat )
     call HistoryGet( trim(adjustl(dat_list(i))), trim(adjustl(vname)),  &
  &                   sst(1:nx,1:ny,1:nz), range=trim(adjustl(forma)) )
     call HistoryGetAttr( trim(adjustl(dat_list(i))), trim(adjustl(vname)),  &
  &                       trim(adjustl(undefchar)), undef )

     call time_zone_convert( int(time(1))*24, itime, otime )
     date(1:4)=trim(adjustl( i2c_convert( otime%year_d ) ))
     date(5:6)=trim(adjustl( i2c_convert( otime%month_d, forma='(i2.2)' ) ))
     date(7:8)=trim(adjustl( i2c_convert( otime%day_d, forma='(i2.2)' ) ))
     date(9:12)='0000'

     oname='fora.sst'//date//'.bin'

     do l=1,nz
        do k=1,ny
           do j=1,nx
              if(sst(j,k,l)/=undef)then
                 sst(j,k,l)=sst(j,k,l)+sst_const
              else
                 sst(j,k,l)=undefc
              end if
           end do
        end do
     end do

!$omp parallel default(shared)
!$omp do schedule(runtime) private(l)

     do l=1,nz
        call auto_interpolation_2d( lon, lat, lond, latd,  &
  &                                 sst(1:nx,1:ny,l), sstd(1:nxd,1:nyd,l),  &
  &                                 undef=undefc, undefr=undefc )
     end do

!$omp end do
!$omp end parallel

     call write_file_3d( trim(adjustl(oname)), nxd, nyd, nzd, 1,  &
  &                      sstd, mode='replace' )
     write(*,*) "Output : "//trim(adjustl(oname))//' ...'

  end do

  write(*,*) "-------------------------------"
  write(*,*) "dset ^fora.sst%y4%m2%d20000.bin"
  write(*,*) "options template big_endian"
  write(*,*) "undef -1.0e35"
  write(*,*) "xdef "//trim(adjustl(i2c_convert(nxd)))//' linear '//  &
  &          trim(adjustl(r2c_convert(lonmin,forma='(f9.5)')))//' '//  &
  &          trim(adjustl(r2c_convert(dlon,forma='(f9.7)')))
  write(*,*) "ydef "//trim(adjustl(i2c_convert(nyd)))//' linear  '//  &
  &          trim(adjustl(r2c_convert(latmin,forma='(f9.5)')))//' '//  &
  &          trim(adjustl(r2c_convert(dlat,forma='(f9.7)')))
  write(*,*) "zdef "//trim(adjustl(i2c_convert(nzd)))//' levels  ', zd
  write(*,*) "tdef   1 linear 00:00Z01Jan0000 1dy"
  write(*,*) "vars   1"
  write(*,*) "sst  "//trim(adjustl(i2c_convert(nzd)))//  &
  &                " 0 sea surface temperature [K]"
  write(*,*) 'endvars'
  write(*,*) "-------------------------------"

  write(*,*) "Finish program."

end program
